/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Martin Rexa
 */

package cz.xelfi.quoridor.statistics.resources;

import cz.xelfi.quoridor.statistics.EloList;
import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.webidor.GameStatus;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 *
 * @author Martin Rexa
 */
public final class Elo {
    private EloList list;
    private ArrayList<EloList> listHistory;
    private boolean firstGame;
    static long day = 1000 * 60 * 60 * 24;

    public Elo(){
        listHistory = new ArrayList<EloList>();
        list = new EloList();
        listHistory.add(list);
        firstGame = true;
    }

    public void processGame(GameId gId){
        GameStatus status = gId.getStatus();
        splitList(gId.getModified());
        if(status.equals(GameStatus.whiteWon)){
            list.putResult(gId.getWhite(), gId.getBlack());
        }else if(status.equals(GameStatus.blackWon)){
            list.putResult(gId.getBlack(), gId.getWhite());
        }
    }

    private void splitList(long gameModified){
        if(firstGame){
            firstGame = false;
            list.setEndDate(computeListEndDate(gameModified));
        }else{
            if(gameModified > list.getEndDate()){
                list = new EloList(list);
                listHistory.add(0,list);
                list.setEndDate(computeListEndDate(gameModified));
            }
        }
    }

    private long computeListEndDate(long modified){
        Calendar now = new GregorianCalendar();
        Calendar cModified = new GregorianCalendar();
        cModified.setTimeInMillis(modified);
        Calendar result = new GregorianCalendar();
        result.clear();
        result.set(Calendar.YEAR, cModified.get(Calendar.YEAR));
        result.set(Calendar.MONTH, cModified.get(Calendar.MONTH)+1);
        return result.getTimeInMillis()-1;
    }

    @GET
    @Path("list")
    @Produces({ MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public EloList getList(){
        return list;
    }

    @GET
    @Path("list/{id}")
    @Produces({ MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public EloList getHistoryList(@PathParam("id") Integer id){
        if(id == null)
            return null;
        if(id >= listHistory.size())
            return null;
        return listHistory.get(id);
    }

    @GET
    @Path("{username}")
    @Produces(MediaType.TEXT_PLAIN)
    public String getElo(@PathParam("username") String id){
        list.getElo(id);
        return Double.toString(list.getElo(id));
    }

}
